module Geo.Gradient
    ( 
	gradient
    ) where


import World.Type
import World.Query.GetFieldSize
import Geo.Type
import Geo.Util
import Geo.Membership (unsegm)
import Monad (when)


-- calc the Path from start to target
-- assert: start or target have to be a homebase
--		the other Pos must be at the border or one step beside the bolder of the homeseqm
gradient :: Info -> DistanceMap ->  Position -> Position -> IO([Position])
gradient info dist start target = do
	-- check home base
	member <- get_membership info dist start
	let (home, peri, outward) = if Geo.Type.distance member == 0 then (start,target, True) else (target,start, False)
	homemember <- get_membership info dist home 
	let home_nr = base_nr homemember
	when (home_nr == unsegm) $ error "Geo.Gradient.gradient: homebase error"
	perimember <-  get_membership info dist peri
	if (base_nr perimember) == home_nr then do
		  -- peri is in the same segm as home
		  ls <- dogradient info dist home_nr peri home 
		  let path = peri:ls
		  return $ if outward then reverse path else path
		else do
			-- peri is not in the same seqm
			newperi <- getnewperi info dist peri home_nr  
			ls <- dogradient info dist home_nr newperi home
			let path =  peri:(newperi:ls)
			return $ if outward then reverse path else path

			
dogradient :: Info -> DistanceMap ->  Int-> Position -> Position -> IO([Position])
dogradient info dist segm pos home = do
  if pos == home then do 
  		return []
  	else do
		let	(width, _ ) = get_field_size info
		let neig = neigb info pos
  		list <- mapM (\ (x',y') -> do 
			cont <- readArray dist (get_index x' y' width)
			return ( Geo.Type.distance cont, (x',y'), base_nr cont ==  segm) 
			) $ neig
		let possibles = filter (\ (_, _, flag) -> flag) list
		when (null possibles)  $ error "Geo.Gradient.dogradient: no following Points"
		let (_,newp,_) = minimum possibles
		prick <- dogradient info dist segm newp home
		return $ newp:prick

-- calc a new peripher point if it is not in the seqm
getnewperi :: Info -> DistanceMap ->  Position -> Int -> IO(Position)			
getnewperi info dist p home_nr = do
	let neig = neigb info p
	let	(width, _ ) = get_field_size info		
	list <- mapM (\ (x',y') -> do 
							  cont <- readArray dist (get_index x' y' width)
							  return ( Geo.Type.distance cont,(x',y'), base_nr cont ==  home_nr) 
					   ) $ neig
	let possibles = filter (\ ( _, _, flag) -> flag) list 
	let (_,newp,_) = minimum possibles
	return newp 
	
neigb :: Info -> Position -> [Position]
neigb info (x,y) = 
	let	(width, height ) = get_field_size info
		neig = [(x+1,y),(x,y+1),(x-1,y),(x,y-1)]
	in filter (\ (x',y') -> and [x'>0, y'>0, x'<=width, y'<=height] ) neig
	   
	
