module Geo.Membership 

( Membership
, membership
, border, base_nr, distance
, unsegm
)

where

-- $Log: Membership.hs,v $
-- Revision 1.4  2002/09/02 11:58:22  joe
-- 32 -> 64
--
-- Revision 1.3  2002/09/02 11:30:04  joe
-- Segmentation durch explizites rumlaufen
--

import Data.Word
import Data.Bits
import GHC.Exts

-- data Membership = Membership { base_nr  :: ! Int
--			     , distance :: ! Int
--			     , border   :: ! Bool
--			     } deriving (Show)

type Membership = Word64

base_width :: Int
base_width = 31

dist_width :: Int
dist_width = 32

total_width :: Int
-- MUST be 64 - 1 since we use Word64
total_width = base_width + dist_width 

-- layout: 2^(base_width + dist_width) * border 
--	   + 2^dist_width * base_nr + distance
--  0 <= dist < 2^dist_width (2^20 is enough for longest maze on 1000 x 1000)
--  0 <= base_nr < 2^base_width ( if larger, Dijkstra is useless anyway ?? )

membership :: Int -> Int -> Bool -> Membership
membership base dist bord = 
    let top = if bord then bit total_width else 0
	bs  = shiftL ( fromIntegral base .&. ( 2^base_width-1 ) ) dist_width
	ds  =        ( fromIntegral dist .&. ( 2^dist_width-1 ) ) 
    in	top .|. bs .|. ds

border :: Membership -> Bool
border m = testBit m total_width

base_nr :: Membership -> Int
base_nr m = fromIntegral $ ( m `shiftR` dist_width ) .&. (2^base_width - 1)

distance :: Membership -> Int
distance m = fromIntegral $ m .&. ( 2^dist_width - 1 )

unsegm :: Int 
unsegm = 2^base_width - 1

