module Geo.Type 
    ( WayGraph (..)
    , Way (..)
    , Graph (..)
    , Edge (..)
    , kante
    , DistanceMap
    , HighwaysFrom
    , Path

    , module Geo.Distance
    , module Geo.Membership

    , module Util.RestrictedFiniteMap
    , module Util.RestrictedSet 
    ) where


import qualified World.Type as W

import Geo.Distance
import Data.Word
import GHC.Exts

import Geo.Distance
import Geo.Membership

import Util.RestrictedSet
import Util.RestrictedFiniteMap

-- $Log: Type.hs,v $
-- Revision 1.30  2002/09/02 14:41:22  michael
-- HighwaysFrom export
--
-- Revision 1.29  2002/09/02 14:39:25  michael
-- HighwaysFrom added
--
-- Revision 1.28  2002/09/02 10:25:23  georg
-- adjacents list in WayGraph
--
-- Revision 1.27  2002/09/02 08:29:48  michael
-- info in graph fixed to path-storage like indeeded
--
-- Revision 1.26  2002/09/02 07:51:31  joe
-- membership as bitstring
--
-- Revision 1.25  2002/09/02 06:37:23  joe
-- Geo distance patch
--
-- Revision 1.24  2002/09/01 23:17:47  joe
-- specialized, strict
--
-- Revision 1.23  2002/09/01 22:57:30  michael
-- Graph type adapted for FiniteMap
--
-- Revision 1.22  2002/09/01 18:06:55  michael
-- W.Position
--
-- Revision 1.21  2002/09/01 18:05:29  michael
-- World.Type
--
-- Revision 1.20  2002/09/01 16:18:55  georg
-- t
--
-- Revision 1.19  2002/09/01 16:02:29  michael
-- InvalidWay
--
-- Revision 1.18  2002/09/01 12:37:47  georg
-- *** empty log message ***
-- 

-------------------------------------------------------------------------------

data WayGraph a  = WayGraph { nodes    :: Set a
			    , edges    :: Set (Edge a)
			    , ewfm     :: FiniteMap (Edge a) (Set Way)
			    , homepos  :: FiniteMap a W.Position
			    , adjacent :: FiniteMap a (Set (Edge a))
			    } deriving (Show)

type HighwaysFrom = ( Int
		    , FiniteMap Int (Int,[(Edge W.Position, Way)])
		    )

data Graph a = Graph { gnodes :: Set a
		     , gedges :: Set (Edge a)
		     , ginfos :: FiniteMap a (Int, [(Edge W.Position,Way)])
		     }

data DistanceGraph a = FiniteMap a Int


-- a path along pos with length
data Way = Way { len :: ! Int
	       , pos :: ! W.Position }
	 | InvalidWay
	   deriving (Eq, Ord, Show)


-- an edge from a to b with several paths to b
data Edge a  = Edge { from  :: ! a
		    , to    :: ! a
		    } deriving (Eq, Ord, Show)

{-# SPECIALIZE kante :: Int -> Int -> Edge Int #-}

kante :: (Ord a) => a -> a -> Edge a
kante f t = if f<t then Edge {from=f, to =t}
				   else Edge {from=t, to =f}


-- a path from one position to another along the
-- main routes
type Path = [ W.Position ]

--patched by Georg
--instance Eq a => Eq (Edge a) where
--    (==) x y = from x == from y

--instance Ord a => Ord (Edge a) where
--    compare x y = compare (from x) (from y)
