module Samples.Kachel where

import IO                     ( readFile , writeFile )
import List                   ( intersperse )
import Monad                  ( guard )
import World
import Random                 ( randomRIO )

------------------------------------------------------------------------------

data Kachel                   =  Kachel [ String ]

instance                         Show Kachel where
    show ( Kachel ss )        =  unlines ss

------------------------------------------------------------------------------

kachel                        :: Int -> IO Kachel
kachel n                      =  do

                                 contents <- readFile $ "Samples/k" ++ show n

                                 return $ Kachel $ lines contents

size                          :: Kachel -> Position
size ( Kachel k )             =  ( length $ head k , length k )

------------------------------------------------------------------------------

beside                        :: Kachel -> Kachel -> Kachel
beside ( Kachel k1 ) ( Kachel k2 )
                              =  Kachel $ zipWith (++) k1 k2

above                         :: Kachel -> Kachel -> Kachel
above ( Kachel k1 ) ( Kachel k2 )
                              =  Kachel $ k1 ++ k2


quadrat                       :: Kachel -> Kachel
quadrat k                     =  above ( beside k k ) ( beside k k )

------------------------------------------------------------------------------

stretch                       :: Int -> Kachel -> Kachel
stretch m ( Kachel k )        =  Kachel $ k'' m

    where k'' 0               =  intersperse line $ k' m
          k'' n               =  intersperse line $ k'' (n-1)
          line                =  replicate ( length $ head $ k' m ) '.'
          k' 0                =  map ( intersperse '.' ) k
          k' n                =  map ( intersperse '.' ) $ k' (n-1)

------------------------------------------------------------------------------

homes                         :: Kachel -> [ Position ]
homes k                       =  do ( ( x , y ) , c ) <- posses k
				    guard $ c == '@'
				    return ( x , y )

posses                        :: Kachel -> [ ( Position , Char ) ]
posses ( Kachel k )           =  do x <- [ 1 .. w ]
				    y <- [ 1 .. h ]
				    return $ ( ( x , y ) 
					     , ( k !! ( h - y ) ) !! (x-1)
					     )
    where ( w , h )           =  size ( Kachel k )

------------------------------------------------------------------------------
                              --   id    from       to         weight
type PackDescr                =  ( Int , Position , Position , Int )

gen_packs                     :: Int -> Int -> Int 
			         -> Kachel -> IO [ PackDescr ]
gen_packs wmin wmax n k       =  do

    let ( w , h )             =  size k
        hs                    =  homes k
        lhs                   =  length hs

    mapM ( \ pid              -> do w <- randomRIO ( wmin , wmax )
                                    h <- randomRIO ( 0 , lhs - 1 )
                                    x <- randomRIO ( -2 , w + 2 )
                                    y <- randomRIO ( -2 , h + 2 )
                                    return ( pid , hs !! h , ( x , y ) , w )
	 ) [1..n]

------------------------------------------------------------------------------

spec                          :: Kachel -> [ PackDescr ] -> String
spec k pds                    =  "[ "
				 ++ ( concat
				    $ intersperse ",\n"
				    $ map ( uncurry spec_base ) bases 
				    )
                                 ++ " ]"
    where bases               =  filter ( (/=[]) . snd ) $ zip bs ( map ps bs )
	  bs                  =  homes k
          ps b                =  filter ( \ (_,from,_,_) -> from == b ) pds


spec_base                     :: Position -> [ PackDescr ] -> String
spec_base base ps             =  "(" ++ show base ++ ", " 
				 ++ spec_packs ps ++ ")"

spec_packs                    :: [ PackDescr ] -> String
spec_packs ps                 =  "[ " 
				 ++ ( concat 
				    $ intersperse ",\n" 
				    $ map spec_pack ps
				    )
				 ++ " ]"

spec_pack                     :: PackDescr -> String
spec_pack ( pid , from , to , w )
                              =  "Mk { uid=" ++ show pid
                                 ++ ", dest=" ++ show from
                                 ++ ", weight=" ++ show w
	                         ++ "}"

------------------------------------------------------------------------------

gen_robots                    :: Int -> Kachel -> IO [ ( RobotID , Position ) ]
gen_robots n k                =  do

    let ( w , h )             =  size k
	
    mapM ( \ rid              -> do x <- randomRIO ( 1 , w )
	                            y <- randomRIO ( 1 , h )
                                    return ( rid , ( x , y ) )
	 ) [1..n]

set_robots                    :: [ ( RobotID , Position ) ] -> String
set_robots                    =  concat . intersperse " " . map set_robot
    where set_robot ( rid , ( x, y ) ) 
                              =  "#" ++ show rid 
			         ++ " X " ++ show x 
				 ++ " Y " ++ show y

------------------------------------------------------------------------------

mk_map                        :: Kachel 
			         -> Int -> Int -> Int
				 -> FilePath
				 -> FilePath
				 -> IO ()
mk_map k wmin wmax n kfile sfile
                              =  do

                                 pds <- gen_packs wmin wmax n k

                                 writeFile kfile ( show k     )
                                 writeFile sfile ( spec k pds )

------------------------------------------------------------------------------

ktest                         :: IO ()
ktest                         =  do
				 [k8,k9,k10,k11] <- mapM kachel [1,1,1,1]
				 let k = quadrat $ quadrat $ quadrat $ quadrat 
					 $ above ( beside k8 k9 )
					         ( beside k10 k11 )
                                 mk_map k 5 150 1000 "Samples/map1" "Samples/spec1"


------------------------------------------------------------------------------


ktest2                         :: IO ()
ktest2                         =  do
				 [k1] <- mapM kachel [30]
				        
                                 mk_map (quadrat k1) 5 150 10 "Samples/map1" "Samples/spec1"

ksimple                        :: IO()
ksimple                        =  do
                                  k1 <- kachel 13
                                  mk_map k1 5 150 10 "Samples/simplemap" "Samples/simplepack"
