module World.Query.Choose where

import World.Type
import qualified Geo.Type as G
import Util.Knapsack
import Util.RestrictedFiniteMap
import Util.RestrictedSet
import World.Query.Robo_Pos
import World.Query.Distance
import qualified Geo.WayFinder as GWF
import Util.Args
import Maybe

-------------------------------------------------------------------------------

choose                        :: Args -> IO ( [ PackID ] , [ PackID ] )
choose args                   =  gen_choose ( GWF.distance args ) ( info args )

-------------------------------------------------------------------------------

simple_choose                 :: Info -> IO ( [ PackID ] , [ PackID ] )
simple_choose                 =  gen_choose ( simple_distanceL )

simple_distanceL              :: Position -> [ Position ] -> IO [ Int ]
simple_distanceL pos poss     =  return $ map ( distance pos ) poss

-------------------------------------------------------------------------------

gen_choose                    :: ( Position -> [ Position ] -> IO [ Int ] )
				 -> Info 
				 -> IO ( [ PackID ] , [ PackID ] )
gen_choose dist_fun i         =  do

    let   dests               =  map ( fst . snd ) packs
          packs               =  map ( \ ( pid , pack ) -> ( pid , dw pack ) )
	                      $  filter ( ( \ p -> p == ( Left    my_pos  ) 
					        || p == ( Right $ my_id i )
					  )
					. pos 
					. snd 
					) 
			      $  fmToList ( packages i )
          dw p                =  ( fromMaybe 
				   ( error "choose: no destination given??" )
				   ( destination p )
				 , fromMaybe
				   ( error "choose: no weight given??" )
				   ( weight p )
				 )
	  my_pos              =  robot_pos ( my_id i ) i

    dists <- dist_fun my_pos dests

    let   my_packids          =  lookupWithDefaultFM
                                  ( robot_packs i )
                                  ( emptySet      )
                                  ( my_id       i )
          me                  =  setToList $ intersect picks my_packids
          not_me              =  setToList $ minusSet  picks my_packids
          picks               =  mkSet $ knapsack w pdws
          w                   =  my_capacity i
          pdws                =  zipWith 
				 ( \ d ( pid , ( _ , w ) ) 
				   -> ( pid , ( d , w ) )
				 ) dists packs

    return ( me , not_me )

