module World.Query.GetNeighbors

( get_neighbors
, valid_pos
)

where

import World.Type
import World.Query.GetFieldSize
import World.Query.Get_Field
import Monad

------------------------------------------------------------------------------

-- finds all positions which are valid and direct neighbors

get_neighbors                 :: ( Map , Info ) -> Position -> IO [ Position ]
get_neighbors (m,i) (x,y)     =  filterM ( valid_pos (m,i) ) l
    where l                   =  [ ( x-1 , y   )
				 , ( x   , y-1 )
				 , ( x+1 , y   )
				 , ( x   , y+1 )
				 ]

-- returns true if position is a Homebase or an open field

valid_pos                     :: ( Map , Info ) -> Position -> IO Bool
valid_pos (m,i) (x,y)
    | x <= 0 || y <= 0 || x > w || y > h
                              =  return False
    | otherwise               =  do 
                                 f <- get_field_with_width m w (x,y)
                                 return $ f `elem` [ Base , Open ]

    where (w,h)               =  get_field_size i
