module World.Update 

( update                      -- Info -> String -> Info
)

where

-- $Log: Update.hs,v $
-- Revision 1.19  2002/09/02 11:24:00  mrahn
-- history
--
-- Revision 1.18  2002/09/02 10:14:43  mrahn
-- new world

import World.Type
import World.Move
import World.Query
import Util.Direction
import Util.RestrictedFiniteMap
import Util.RestrictedSet

------------------------------------------------------------------------------

update                        :: Info -> String -> Info
update i s                    =  new_info
    where ( rids , pre_info ) =  update' i ( words s )
          alive               =  flip elementOf rids
          new_info            =  pre_info
              { robot_packs   =  filterFM ( \ rid _ -> alive rid ) 
	                         $ robot_packs pre_info
              , robot_poss    =  filterFM ( \ rid _ -> alive rid ) 
	                         $ robot_poss  pre_info
	      , packages      =  filterFM 
		                 (  \ _ p     -> case pos p of
                                    Left _    -> True
                                    Right rid -> alive rid
				 )
		                 $ packages    pre_info
	      }

------------------------------------------------------------------------------

update'                       :: Info -> [ String ] -> ( Set RobotID , Info )
update' i (('#':rid):ss)      =  update_id (         new_rid     )
                                           (         my_id     i )
                                           (         r_pos       ) 
                                           ( unitSet new_rid , i ) 
                                           ss
    where new_rid             =  read rid
	  r_pos               =  robot_pos new_rid i

update' _ _                   =  error "incorrect update-string"

------------------------------------------------------------------------------

update_id                     :: RobotID 
                                 -> RobotID
			         -> Position
				 -> ( Set RobotID , Info ) 
				 -> [ String ] 
				 -> ( Set RobotID , Info )

--
update_id _ _ _ (rids,i) []   =  ( rids , i )                     


--
update_id _ our_id _ (rids,i) (('#':rid):ss)
                              =  update_id new_id our_id new_pos
                                           ( addToSet rids new_id , i ) 
                                           ss
    where new_id              =  read rid
          new_pos             =  robot_pos new_id i

--
update_id rid our_id r_pos (rids,i) ("N":ss) 
                              =  update_id rid our_id new_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i 
	      { robot_poss    =  addToFM ( robot_poss i ) rid new_pos
 	      , our_actions   =  if rid == our_id 
		                 then add_action ( Move N ) ( our_actions i )
		                 else our_actions i
	      }
          new_pos             =  move N r_pos

--
update_id rid our_id r_pos (rids,i) ("S":ss)  
                              =  update_id rid our_id new_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i 
	      { robot_poss    =  addToFM ( robot_poss i ) rid new_pos
 	      , our_actions   =  if rid == our_id 
		                 then add_action ( Move S ) ( our_actions i )
		                 else our_actions i
	      }
          new_pos             =  move S r_pos

--
update_id rid our_id r_pos (rids,i) ("E":ss)
                              =  update_id rid our_id new_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i 
	      { robot_poss    =  addToFM ( robot_poss i ) rid new_pos
 	      , our_actions   =  if rid == our_id 
		                 then add_action ( Move E ) ( our_actions i )
		                 else our_actions i
	      }
          new_pos             =  move E r_pos

--
update_id rid our_id r_pos (rids,i) ("W":ss)    
                              =  update_id rid our_id new_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i 
	      { robot_poss    =  addToFM ( robot_poss i ) rid new_pos
 	      , our_actions   =  if rid == our_id 
		                 then add_action ( Move W ) ( our_actions i )
		                 else our_actions i
	      }
          new_pos             =  move W r_pos

--
update_id rid our_id r_pos (rids,i) ("P":packid:ss)
                              =  update_id rid our_id r_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i
              { robot_packs   =  addToFM_C ( union              ) 
                                           ( robot_packs i      ) 
                                             rid 
                                           ( unitSet new_packid )
	      , packages      =  addToFM ( packages i ) 
                                           new_packid
                                           the_new_pack
 	      , our_actions   =  if rid == our_id 
		                 then add_action ( Pick [ new_packid ] ) 
                                                 ( our_actions i )
		                 else our_actions i
	      }
  
          new_packid          =  read packid
  
          the_new_pack        =  case lookupFM ( packages i ) new_packid of
                      Nothing -> new_pack
		      Just p  -> new_pack
				 { weight      = weight p
				 , destination = destination p
				 }

          new_pack            =  Package
              { weight        =  Nothing
	      , destination   =  Nothing
	      , pos           =  Right rid
	      , was_dropped   =  False
	      }

--
update_id rid our_id r_pos (rids,i) ("D":packid:ss)
                              =  update_id rid our_id r_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i
              { robot_packs   =  addToFM ( robot_packs i ) 
                                         rid 
                                         ( mkSet new_ids )
	      , packages      =  if is_destination 
		                 then delFromFM ( packages i ) the_packid
		                 else addToFM ( packages i ) 
                                              the_packid
                                              the_new_pack
	      , my_score      =  if is_destination && rid == our_id
		                 then my_score i + the_weight
		                 else my_score i
 	      , our_actions   =  if rid == our_id 
		                 then add_action ( Drop [ the_packid ] ) 
                                                 ( our_actions i )
		                 else our_actions i
	      }

          ( the_new_pack , is_destination , the_weight )
                              =  case lookupFM ( packages i ) the_packid of
                      Nothing -> ( new_pack 
				 , False 
				 , error "update: requested weight undefined" 
				 )
		      Just p  -> ( new_pack
				   { weight      = weight      p
				   , destination = destination p
				   }
				 , case destination p of
				    Nothing    -> False
				    Just (x,y) -> (x,y) == r_pos
				 , case weight p of
                                    Nothing -> 0
				    Just w  -> w
				 )

          the_packid          =  read packid

          new_pack            =  Package
	      { weight        =  Nothing
	      , destination   =  Nothing
	      , pos           =  Left $ r_pos
	      , was_dropped   =  rid /= our_id
	      }

          new_ids             =  filter (/=the_packid) $ robot_pack rid i


--
update_id rid our_id r_pos (rids,i) ("X":xpos:"Y":ypos:ss)
                              =  update_id rid our_id new_pos 
                                           ( rids , new_info ) ss
    where new_info            =  i
              { robot_poss    =  addToFM ( robot_poss i ) 
		                         rid
                                         new_pos
	      }
          new_pos             =  ( read xpos , read ypos ) 

update_id  _ _ _ _ _          =  error "incorrect update-string"
